/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.farm;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.logging.Level;
import java.util.logging.Logger;
import nl.cloudfarming.client.farm.model.Farm;
import nl.cloudfarming.client.farm.model.FarmManager;
import org.netbeans.api.project.Project;
import org.netbeans.api.project.ProjectManager;
import org.netbeans.spi.project.ui.support.DefaultProjectOperations;
import org.openide.filesystems.FileLock;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.util.Exceptions;
import org.openide.util.lookup.ServiceProvider;

/**
 * Stateless
 * @author Timon Veenstra
 */
@ServiceProvider(service = FarmManager.class)
public class FarmManagerImpl implements FarmManager {

    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.farm");

    /**
     * Load a Farm from a file object
     *
     * @param fileObject
     * @return
     */
    @Override
    public Farm load(FileObject fileObject) {
        Farm farm = null;
        try (ObjectInputStream ois = new ObjectInputStream(fileObject.getInputStream())) {
            farm = (Farm) ois.readObject();
        } catch (ClassNotFoundException | IOException ex) {
            Exceptions.printStackTrace(ex);
        }
        return farm;
    }

    /**
     *
     * @param farm
     * @param fileObject
     */
    @Override
    public void save(Farm farm, FileObject fileObject) {
        try (ObjectOutputStream oos = new ObjectOutputStream(fileObject.getOutputStream())) {
            oos.writeObject(farm);
        } catch (IOException ex) {
            Exceptions.printStackTrace(ex);
        }
        //
        // renaming project folder if farm name changed
        //
        try {
            FileObject projectDir = fileObject.getParent();

            if (!projectDir.getName().equals(farm.getName())) {
                LOGGER.log(Level.FINEST, "Farm name changed to: {0}", farm.getName());

                String actualNewName = FileUtil.findFreeFolderName(projectDir.getParent(), farm.getName());
                LOGGER.log(Level.FINEST, "Action new free name determined to be: {0}", actualNewName);
//            projectDir.rename(FileLock.NONE, actualNewName, null);
                Project project = ProjectManager.getDefault().findProject(projectDir);
                DefaultProjectOperations.performDefaultRenameOperation(project, actualNewName);
            }
        } catch (IOException ex) {
            Exceptions.printStackTrace(ex);
        }
    }
}
