/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.farm;

import java.awt.datatransfer.Transferable;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import org.netbeans.api.project.Project;
import org.netbeans.spi.project.SubprojectProvider;
import org.netbeans.spi.project.ui.LogicalViewProvider;
import org.netbeans.spi.project.ui.support.NodeFactory;
import org.netbeans.spi.project.ui.support.NodeFactorySupport;
import org.netbeans.spi.project.ui.support.NodeList;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.ChildFactory;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import org.openide.util.NbBundle;
import org.openide.util.datatransfer.PasteType;

/**
 *
 * @author Timon Veenstra
 */
@NodeFactory.Registration(projectType = "nl-cloudfarming-client-farm")
public class FarmSubProjectsNodeFactory implements NodeFactory {

    static String REGISTERED_NODE_LOCATION = "Projects/nl-cloudfarming-client-farm/Nodes";
    private static final Logger LOG = Logger.getLogger("nl.cloudfarming.client.farm");

    @Override
    public NodeList createNodes(Project project) {
        return NodeFactorySupport.fixedNodeList(new ContainerNode(project));
    }

    @NbBundle.Messages(value="container node name=Containers")
    private class ContainerNode extends AbstractNode {
        public ContainerNode(Project project) {
            super(Children.create(new ContainerNodeChildFactory(project), true));
            setDisplayName(Bundle.container_node_name());
        }
        
        @Override
        public PasteType getDropType(Transferable t, int action, int index) {
            return null;
        }
    }

    private static class ContainerNodeChildFactory extends ChildFactory<Project> {

        private final Project project;
        private final SubprojectProvider subprojectProvider ;

        public ContainerNodeChildFactory(Project project) {
            this.project = project;
            this.subprojectProvider = project.getLookup().lookup(SubprojectProvider.class);
            this.subprojectProvider.addChangeListener(new ChangeListener() {

                @Override
                public void stateChanged(ChangeEvent e) {
                    refresh(true);
                }
            });
        }

        @Override
        protected boolean createKeys(List<Project> toPopulate) {
            if (subprojectProvider != null) {
                toPopulate.addAll(subprojectProvider.getSubprojects());
            } else {
                LOG.log(Level.WARNING, "SubprojectProvider expected in the lookup of Farm project {0}", project.toString());
            }
            return true;
        }

        @Override
        protected Node createNodeForKey(Project key) {
            return key.getLookup().lookup(LogicalViewProvider.class).createLogicalView();
        }
        
        
    }
}
